
#include "crypto_core_hsalsa20.h"

#define crypto_core crypto_core_hsalsa20
#define crypto_core_OUTPUTBYTES crypto_core_hsalsa20_OUTPUTBYTES
#define crypto_core_INPUTBYTES crypto_core_hsalsa20_INPUTBYTES
#define crypto_core_KEYBYTES crypto_core_hsalsa20_KEYBYTES
#define crypto_core_CONSTBYTES crypto_core_hsalsa20_CONSTBYTES
#define crypto_core_IMPLEMENTATION crypto_core_hsalsa20_IMPLEMENTATION
#define crypto_core_VERSION crypto_core_hsalsa20_VERSION
