
#include "crypto_core_salsa208.h"

#define crypto_core crypto_core_salsa208
#define crypto_core_OUTPUTBYTES crypto_core_salsa208_OUTPUTBYTES
#define crypto_core_INPUTBYTES crypto_core_salsa208_INPUTBYTES
#define crypto_core_KEYBYTES crypto_core_salsa208_KEYBYTES
#define crypto_core_CONSTBYTES crypto_core_salsa208_CONSTBYTES
#define crypto_core_IMPLEMENTATION crypto_core_salsa208_IMPLEMENTATION
#define crypto_core_VERSION crypto_core_salsa208_VERSION
