
#include "crypto_hash_sha256.h"

#define crypto_hash crypto_hash_sha256
#define crypto_hash_init crypto_hash_sha256_init
#define crypto_hash_update crypto_hash_sha256_update
#define crypto_hash_final crypto_hash_sha256_final
#define crypto_hash_BYTES crypto_hash_sha256_BYTES
#define crypto_hash_IMPLEMENTATION crypto_hash_sha256_IMPLEMENTATION
#define crypto_hash_VERSION crypto_hash_sha256_VERSION
