
#include "crypto_stream_salsa208.h"

#define crypto_stream crypto_stream_salsa208
#define crypto_stream_xor crypto_stream_salsa208_xor
#define crypto_stream_KEYBYTES crypto_stream_salsa208_KEYBYTES
#define crypto_stream_NONCEBYTES crypto_stream_salsa208_NONCEBYTES
#define crypto_stream_IMPLEMENTATION crypto_stream_salsa208_IMPLEMENTATION
#define crypto_stream_VERSION crypto_stream_salsa208_VERSION
